@@ -9,6 +9,7 @@ import re
 from functools import partial
 from gettext import gettext as _
 from weakref import WeakValueDictionary
+from contextlib import suppress
 
 from .child import cached_process_data
 from .cli import create_opts, parse_args
@@ -28,7 +29,7 @@ from .fast_data_types import (
     toggle_maximized
 )
 from .keys import get_shortcut, shortcut_matches
-from .layout import set_draw_borders_options
+from .layout import set_layout_options
 from .remote_control import handle_cmd
 from .rgb import Color, color_from_int
 from .session import create_sessions
@@ -105,7 +106,7 @@ class DumpCommands:  # {{{
 class Boss:
 
     def __init__(self, os_window_id, opts, args, cached_values, new_os_window_trigger):
-        set_draw_borders_options(opts)
+        set_layout_options(opts)
         self.clipboard_buffers = {}
         self.update_check_process = None
         self.window_id_map = WeakValueDictionary()
@@ -134,13 +135,13 @@ class Boss:
         if new_os_window_trigger is not None:
             self.keymap.pop(new_os_window_trigger, None)
         for startup_session in startup_sessions:
-            os_window_id = self.add_os_window(startup_session, os_window_id=os_window_id)
+            self.add_os_window(startup_session, os_window_id=os_window_id)
+            os_window_id = None
             if args.start_as != 'normal':
                 if args.start_as == 'fullscreen':
                     self.toggle_fullscreen()
                 else:
                     change_os_window_state(args.start_as)
-            os_window_id = None
         if is_macos:
             from .fast_data_types import cocoa_set_notification_activated_callback
             cocoa_set_notification_activated_callback(self.notification_activated)
@@ -639,16 +640,12 @@ class Boss:
     def notify_on_os_window_death(self, address):
         import socket
         s = socket.socket(family=socket.AF_UNIX)
-        try:
+        with suppress(Exception):
             s.connect(address)
             s.sendall(b'c')
-            try:
+            with suppress(EnvironmentError):
                 s.shutdown(socket.SHUT_RDWR)
-            except EnvironmentError:
-                pass
             s.close()
-        except Exception:
-            pass
 
     def display_scrollback(self, window, data, cmd):
         tab = self.active_tab
@@ -760,10 +757,10 @@ class Boss:
     def kitty_shell(self, window_type):
         cmd = ['@', kitty_exe(), '@']
         if window_type == 'tab':
-            self._new_tab(cmd).active_window
+            self._new_tab(cmd)
         elif window_type == 'os_window':
             os_window_id = self._new_os_window(cmd)
-            self.os_window_map[os_window_id].active_window
+            self.os_window_map[os_window_id]
         elif window_type == 'overlay':
             w = self.active_window
             tab = self.active_tab
@@ -913,6 +910,11 @@ class Boss:
                 tm.new_tab(special_window=create_window(), cwd_from=cwd_from)
         elif dest == 'os_window':
             self._new_os_window(create_window(), cwd_from=cwd_from)
+        elif dest in ('clipboard', 'primary'):
+            env, stdin = self.process_stdin_source(stdin=source, window=window)
+            if stdin:
+                func = set_clipboard_string if dest == 'clipboard' else set_primary_selection
+                func(stdin)
         else:
             import subprocess
             env, stdin = self.process_stdin_source(stdin=source, window=window)
@@ -1032,18 +1034,14 @@ class Boss:
 
     def safe_delete_temp_file(self, path):
         if is_path_in_temp_dir(path):
-            try:
+            with suppress(FileNotFoundError):
                 os.remove(path)
-            except FileNotFoundError:
-                pass
 
     def set_update_check_process(self, process=None):
         if self.update_check_process is not None:
-            try:
+            with suppress(Exception):
                 if self.update_check_process.poll() is None:
                     self.update_check_process.kill()
-            except Exception:
-                pass
         self.update_check_process = process
 
     def on_monitored_pid_death(self, pid, exit_status):
