@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # vim:fileencoding=utf-8
 # License: GPL v3 Copyright: 2016, Kovid Goyal <kovid at kovidgoyal.net>
 
@@ -9,6 +9,7 @@
 # will replay the commands and pause at the end waiting for user to press enter
 
 import sys
+from contextlib import suppress
 
 
 CSI = '\033['
@@ -86,6 +87,10 @@ def screen_erase_in_line(how, private):
     write(CSI + ('?' if private else '') + str(how) + 'K')
 
 
+def screen_delete_lines(num):
+    write(CSI + str(num) + 'M')
+
+
 def screen_cursor_up2(count):
     write(CSI + '%dA' % count)
 
@@ -118,6 +123,10 @@ def draw(*a):
     write(' '.join(a))
 
 
+def screen_manipulate_title_stack(op, which):
+    write(CSI + '%d;%dt' % (op, which))
+
+
 def report_device_attributes(mode, char):
     x = CSI
     if char:
@@ -150,7 +159,5 @@ def replay(raw):
 def main(path):
     raw = open(path).read()
     replay(raw)
-    try:
+    with suppress(EOFError, KeyboardInterrupt):
         input()
-    except (EOFError, KeyboardInterrupt):
-        pass
